import escapeRegex from "escape-string-regexp";
export const NO_INFO = null;
export function verificationRegexFor(r) {
    return join(r.contexts.map(c => new RegExp(c.source.replace(escapeRegex(r.change.from), r.change.to))));
}
export function mistakesRegexFor(r) {
    return join(r.contexts);
}
export function proofreadWith(c) {
    function replacer(mark, substring) {
        const transform = (substring === undefined
            ? mark
            : (s) => s.split(substring).join(mark(substring)));
        return (match, ...rest) => {
            const captured = rest.find(x => x !== undefined);
            return (typeof captured === "string"
                ? match.replace(captured, transform(captured))
                : transform(match));
        };
    }
    const markWith = c.markWith;
    const { mistake, verified } = c.identifiers;
    const transformers = c.rules.map(r => compose((text) => text.replace(verificationRegexFor(r), replacer(markWith(verified)(NO_INFO), r.change.to)), (text) => text.replace(mistakesRegexFor(r), replacer(markWith(mistake)(r.info || NO_INFO), r.change.from))));
    return transformers.reduceRight(compose, x => x);
}
export function highlightWith(c) {
    return text => text.replace(c.pattern, c.mark);
}
export function compose(f, g) {
    return x => f(g(x));
}
export function join(patterns) {
    return new RegExp(patterns.map(p => p.source).join("|"), "g");
}
//# sourceMappingURL=core.js.map